@@ -58,8 +58,8 @@ class Account(Payment):
         return super().charge(feature_name, accountDetails, requiredParameters, endpoint)
 
     def validate(self, flwRef, otp):
-        endpoint = self._baseUrl + self._endpointMap['account']['verify']
-        feature_name = "Account-charge-verify"
+        endpoint = self._baseUrl + self._endpointMap['account']['validate']
+        feature_name = "Account-charge-validate"
         return super().validate(feature_name, flwRef, endpoint)
 
     def verify(self, txRef):
